#! /bin/bash  
# patch the SD/CF card bootsector to make it bootable on Canon DSLR
# See http://chdk.setepontos.com/index.php/topic,4214.0.html
#     http://en.wikipedia.org/wiki/File_Allocation_Table#Boot_Sector



if [[ $OSTYPE == darwin* ]]; then
# show a prompt that ask for the card (added by scrax, tested on mac osx only)
	while [ "$dev" = "" ]; do
		echo 
		printf "drop your card from finder here => "
		read mountpoint
		echo
		if [ "$mountpoint" != "" ]; then
		  echo "Mount Point: $mountpoint"
		  if [ -f "$mountpoint" ]; then
		    echo
			echo "========================"
		    echo "You have to drop the SD "
		    echo " card icon, not a file. "
		    echo "========================"
		    echo
		  else
			dev=$(diskutil info $mountpoint | grep 'Device Node' | awk '{print $3}')
		  fi
		else
			echo "========================"
			echo "   press enter to exit  "
			echo " any other key to restart"
			printf "========================\n"
			read exit
			if [ "$exit" = "" ];then
				exit
			fi
		fi
    done
	echo "You device node is: $dev"
	echo "========================"
	echo "Check if this is correct"
	echo "on the list you will see"
	echo "   under this message   "
	echo "========================"
	diskutil list
	echo
	printf "to continue type: y for yes "
	read continue
	if [ "$continue" != "y" ];then
	  exit
	fi
else 
# change this on linux
dev=/dev/sdb1
fi

# read the boot sector to determine the filesystem version
DEV32=`dd if=$dev bs=1 skip=82 count=8`
DEV16=`dd if=$dev bs=1 skip=54 count=8`
if [ "$DEV16" != 'FAT16   ' -a "$DEV32" != 'FAT32   ' ]; then
  echo "Error: "$dev" is not a FAT16 or FAT32 device"
  echo debug $dev $DEV16 $DEV32
  exit
fi
if [ "$DEV16" = 'FAT16   ' ]; then
  offset1=43
  offset2=64
  FS='FAT16'
elif [ "$DEV32" = 'FAT32   ' ]; then
  offset1=71
  offset2=92
  FS='FAT32'
else
  echo "Error: "$dev" is not a FAT16 or FAT32 device"
  exit
fi
echo "Applying "$FS" parameters on "$dev" device:"
echo " writing EOS_DEVELOP at offset" $offset1 "(Volume label)"
echo EOS_DEVELOP | dd of="$dev" bs=1 seek=$offset1 count=11
echo " writing BOOTDISK at offset" $offset2 "(Boot code)"
echo BOOTDISK | dd of="$dev" bs=1 seek=$offset2 count=8
